
# q-expansion of newform 6720.2.a.bs, downloaded from the LMFDB on 02 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 6720
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [1], [-1], [1], [2], [-4], [2], [-2], [-4], [2], [6], [6], [6], [-4], [0], [-8], [0], [10], [-12], [14], [4], [8], [12], [-14], [8], [6], [8], [-12], [-2], [12], [16], [-4], [16], [14], [22], [20], [-24], [-4], [8], [-18], [10], [-18], [-14], [2], [12], [-18], [12], [-20], [28], [-14], [-4], [6], [-2], [24], [-2], [-8], [30], [2], [-10], [6], [4], [30], [-16], [8], [-24], [8], [36], [18], [8], [-14], [-14], [18], [-4], [34], [16], [-16], [14], [-12], [34], [-14], [-28], [-22], [6], [-8], [14], [24], [-22], [-26], [-38], [24], [36], [-8], [32], [18], [24], [32], [6], [-18], [-8], [-38], [12], [0], [-4], [26], [-32], [16], [12], [-6], [-30], [-26], [-4], [30], [20], [38], [-8], [46], [-16], [-24], [-20], [6], [34], [6], [-34], [-24], [-6], [-30], [26], [-20], [28], [-28], [-40], [-24], [-40], [2], [-38], [-26], [38], [8], [6], [-10], [22], [-22], [-24], [-44], [6], [4], [-28], [-34], [-14], [-44], [22], [2], [-4], [16], [4], [58], [8], [34], [-44], [-30], [-32], [-4], [-16], [-32], [36], [-24], [-16], [-4], [-50], [-18], [10], [38], [62], [34], [10], [54], [28], [30], [-60], [-50], [0], [48], [-46], [-62], [24], [6], [-26], [0], [14], [12], [8], [-52], [48], [18], [40], [42], [14], [-14], [46], [-24], [18], [-34], [-32], [46], [0], [22], [-22], [-36], [-50], [2], [-10], [-66], [32], [28], [-44], [-50], [-40], [-30], [-32], [-60], [58], [-42], [30], [-64], [44], [54], [-18], [-30], [-64], [-22], [-46], [-26], [44], [46], [24], [-8], [-30], [12], [58], [-68], [-60], [46], [0], [22], [-66], [60], [12], [-20], [20], [-44], [14], [2], [-20], [-30], [14], [10], [-10], [-44], [-2], [-64], [-16], [0], [34], [16], [18], [-62], [26], [30], [-76], [36], [-34], [28], [-16], [-32], [-8], [44], [-44], [-62], [58], [40], [56], [40], [-48], [54], [8], [50], [62], [-64], [46], [50], [-22], [76], [60], [28], [42], [-54], [-54], [26], [16], [32], [4], [-4], [-12], [8], [-48], [-62], [-36], [-54], [54], [66], [-72], [26], [-30], [-4], [-20], [-58], [12], [-54], [-40], [42], [-14], [70], [-62], [16], [-88], [78], [-32], [-44], [-60], [-64], [62], [-28], [82], [-84], [68], [68], [-70], [-66], [-50], [52], [-6], [-92], [86], [68], [-60], [22], [-30], [60], [36], [-2], [-18], [-50], [-78], [16], [-66], [-62], [84], [-36], [12], [64], [-38], [6], [72], [-90], [-40], [54], [-14], [16], [-6], [-30], [10], [-24], [78], [90], [-34], [-84], [-6], [24], [-74], [-28], [-2], [92], [64], [8], [-50], [24], [72], [-28], [20], [80], [-26], [-54], [54], [-40], [86], [26], [26], [-6], [-8], [66], [-74], [36], [-80], [-10], [74], [-30], [62], [-86], [-4], [32], [-20], [-96], [48], [-4], [58], [14], [38], [44], [58], [-48], [70], [28], [24], [-56], [40], [94], [100], [-30], [34], [52], [34], [22], [60], [-56], [40], [78], [106], [54], [108]]
